#!/usr/bin/env python
#
# Copyright (c) 2016-2020, Virtuozzo International GmbH
#
# This file is part of disp-helper. It's free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Our contact details: Virtuozzo International GmbH, Vordergasse 59, 8200
# Schaffhausen, Switzerland.
#

import re
import time
import os
import gzip
from infra_cep_helper import CEPCollecting


migration_status = {'20Task_MigrateCtSource': {'failed': [], 'completed': []},
                    '20Task_MigrateVmSource': {'failed': [], 'completed': []}}

# add here new errors
migration_bad = {'PRL_ERR_VM_ALREADY_REGISTERED_VM_NAME': [],
                 'PRL_ERR_CT_MIGRATE_INTERNAL_ERROR': []}
# migration types, add here new types
migration_types = ['20Task_MigrateVmSource', '20Task_MigrateCtSource']
migration_fail = ['PRL_ERR_CT_MIGRATE_INTERNAL_ERROR',
                  'PRL_ERR_VM_ALREADY_REGISTERED_VM_NAME']

success = '(.*?)(PRL_ERR_SUCCESS)'


class Migration(CEPCollecting):
    def get_logs(self):
        path = '/var/log/'
        r = re.compile("prl-disp.log*")
        past = time.time() - 7 * 86400  # 7 days
        log_files = [f for f in os.listdir(path) if os.stat(os.path.join(path, f)).st_mtime >= past]
        return list(filter(r.match, log_files))

    def find_migration(self):
        week_logs = self.get_logs()
        for log_file in week_logs:
            if log_file.endswith('.gz'):
                opener = gzip.open
            else:
                opener = open
            try:
                with opener("/var/log/" + log_file, "r") as f:
                    for l in f.readlines():
                        # collect success migrations
                        for m_type in migration_types:
                            for fail_reason in migration_fail:
                                migration_failed = re.findall(m_type + '(.*?)' + fail_reason, l.strip())
                                for item in migration_failed:
                                    migration_status[m_type]['failed'].append("{}{}".format(item[0:].strip("'"), fail_reason))
                            migration_success = re.findall(m_type + success, l.strip())
                            for item in migration_success:
                                migration_status[m_type]['completed'].append("{}{}".format(item[0].strip("'"), item[1]))
            except IOError:
                return {}
        return migration_status

    def fill_results(self):
        self.result["Migration"] = self.find_migration()


def main():
    o = Migration()
    o.run()


if __name__ == "__main__":
    main()
